| Copyright | (c) Sirui Lu 2024 |
|---|---|
| License | BSD-3-Clause (see the LICENSE file) |
| Maintainer | siruilu@cs.washington.edu |
| Stability | Experimental |
| Portability | GHC only |
| Safe Haskell | Safe-Inferred |
| Language | Haskell2010 |
Grisette.Internal.Unified.Class.UnifiedRep
Description
Synopsis
- class UnifiedConRep a where
- type ConType a
- class UnifiedSymRep a where
- type SymType a
Documentation
class UnifiedConRep a Source #
A class that gives the concrete type of a unified primitive type.
Instances
class UnifiedSymRep a Source #
A class that gives the symbolic type of a unified primitive type.